//
//  ClipEditSlider.m
//  igroup
//

#import "ClipEditSlider.h"

#define THUMB_WIDTH 18.0

@implementation ClipEditSlider

@synthesize thumbLeft, thumbRight;
@synthesize activeThumb;
@synthesize activeTrack, trackBase;
@synthesize valueLeft, valueRight;
@synthesize minimumValue, maximumValue;

- (void)awakeFromNib {
	trackBase.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"track_inactive.png"]];
	activeTrack.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"track_active.png"]];
	trackMin = -THUMB_WIDTH / 2;
	trackMax = self.frame.size.width + THUMB_WIDTH / 2;
	minimumX = trackMin + THUMB_WIDTH;
	maximumX = trackMax - THUMB_WIDTH;
	
	/* Set default values. */
	leftX = minimumX;
	rightX = maximumX;
	
	valueLeft = self.minimumValue = 0.0;
	valueRight = self.maximumValue = 100.0;
	
	[self updateActiveTrack];
	
	/* Reset the thumbs to the leftmost and rightmost values. */
	[self setOriginX:trackMin ofThumb:thumbLeft];
	[self setOriginX:(trackMax - THUMB_WIDTH) ofThumb:thumbRight];
}

- (void)setOriginX:(double)x ofThumb:(UIImageView *)thumb {
	thumb.frame = CGRectMake(x, thumb.frame.origin.y, THUMB_WIDTH, thumb.frame.size.height);
}

#pragma mark Override getters

- (double)valueLeft {
	double percentLeft = (leftX - minimumX) / (maximumX - minimumX);
	return minimumValue + (percentLeft * (maximumValue - minimumValue));
}

- (double)valueRight {
	double percentRight = (rightX - minimumX) / (maximumX - minimumX);
	return minimumValue + (percentRight * (maximumValue - minimumValue));
}

- (void)setValueLeft:(double)newValueLeft {
	assert(newValueLeft >= minimumValue && newValueLeft <= self.valueRight);
	
	double percentLeftOfValue = ((newValueLeft - minimumValue) / (maximumValue - minimumValue));
	leftX = (minimumX + (percentLeftOfValue * (maximumX - minimumX)));
	[self setOriginX:(leftX - THUMB_WIDTH) ofThumb:thumbLeft];
	
	[self updateActiveTrack];
}

- (void)setValueRight:(double)newValueRight {
	assert(newValueRight >= self.valueLeft && newValueRight <= maximumValue);
	
	double percentRightOfValue = ((newValueRight - minimumValue) / (maximumValue - minimumValue));
	rightX = minimumX + (percentRightOfValue * (maximumX - minimumX));
	[self setOriginX:rightX ofThumb:thumbRight];
	
	[self updateActiveTrack];
}

#pragma mark Touch handlers

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {	
	UITouch *touch = [touches anyObject];
	
	if ([self touch:touch isThumb:thumbLeft]) {
		self.activeThumb = thumbLeft;
	} else if ([self touch:touch isThumb:thumbRight]) {
		self.activeThumb = thumbRight;
	}
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	if (!activeThumb) return;
	
	CGPoint point = [[touches anyObject] locationInView:self];
	
	if ([self activeThumbPointIsValid:point.x]) {
		[self updateActiveThumbPosition:point.x];
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	[activeThumb release];
	activeThumb = nil;
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
	[self touchesEnded:touches withEvent:event];
}

#pragma mark Touch handler helpers

- (void)updateActiveThumbPosition:(double)centerX {
	CGPoint oldCenter = activeThumb.center;
	activeThumb.center = CGPointMake(centerX, oldCenter.y);
	if (activeThumb == thumbLeft) {
		leftX = activeThumb.frame.origin.x + THUMB_WIDTH;
	} else if (activeThumb == thumbRight) {
		rightX = activeThumb.frame.origin.x;
	} else {
		assert(NO);
	}
	
	[self sendActionsForControlEvents:UIControlEventValueChanged];
	
	[self updateActiveTrack];
}

- (void)updateActiveTrack {
	activeTrack.frame = CGRectMake(leftX, activeTrack.frame.origin.y, rightX - leftX, activeTrack.frame.size.height);
}

- (BOOL)touch:(UITouch *)touch isThumb:(UIImageView *)thumb {
	CGPoint point = [touch locationInView:thumb];
	return [thumb pointInside:point withEvent:nil];
}
		
- (BOOL)activeThumbPointIsValid:(double)centerX {
	if (!activeThumb) return NO;
	
	double leftEnd, rightEnd;
	if (activeThumb == thumbLeft) {
		leftEnd = trackMin;
		rightEnd = thumbRight.frame.origin.x;
	} else if (activeThumb == thumbRight) {
		leftEnd = thumbLeft.frame.origin.x + THUMB_WIDTH;
		rightEnd = trackMax;
	} else {
		assert(NO);
	}
	
	double thumbRadius = THUMB_WIDTH / 2;
	return (centerX - thumbRadius >= leftEnd) && (centerX + thumbRadius <= rightEnd);
}

- (void)dealloc {
	[activeThumb release];
    [super dealloc];
}


@end
